-- The image to set to on right click, for this pack it's the x on a gray background
local OnRightClickMapImage = "overlay|images/compactlocations/nothing.png"

-- The amount of separate compact map squares to create, will be created as compactmap(one through set count)
local CompactMapLocationCount = 128

-- Define items to add to the palette by adding the necesarry data here
-- Attributes set are similar to what you'd do when defining items in json, just slightly less verbose
local PaletteData = {
	-- Define default square here, this is what any compact map square will default to, or revert to when the applied palette is removed
	[0] = {
		["name"] = "Blank",
		["code"] = "paletteblank",
		["image"] = "images/compactlocations/blank.png"
	},

	-- Dungeons
	{
		["name"] = "Level 1",
		["code"] = "palettelevel1",
		["image"] = "images/compactlocations/levelone.png"
	},
	
	{
		["name"] = "Level 2",
		["code"] = "palettelevel2",
		["image"] = "images/compactlocations/leveltwo.png"
	},

	{
		["name"] = "Level 3",
		["code"] = "palettelevel3",
		["image"] = "images/compactlocations/levelthree.png"
	},
	
	{
		["name"] = "Level 4",
		["code"] = "palettelevel4",
		["image"] = "images/compactlocations/levelfour.png"
	},
	
	{
		["name"] = "Level 5",
		["code"] = "palettelevel5",
		["image"] = "images/compactlocations/levelfive.png"
	},	
	
	{
		["name"] = "Level 6",
		["code"] = "palettelevel6",
		["image"] = "images/compactlocations/levelsix.png"
	},

	{
		["name"] = "Level 7",
		["code"] = "palettelevel7",
		["image"] = "images/compactlocations/levelseven.png"
	},
	
	{
		["name"] = "Level 8",
		["code"] = "palettelevel8",
		["image"] = "images/compactlocations/leveleight.png"
	},

	{
		["name"] = "Level 9",
		["code"] = "palettelevel9",
		["image"] = "images/compactlocations/levelnine.png"
	},
	
	{
		["name"] = "Level Unknown",
		["code"] = "palettelevelunknown",
		["image"] = "images/compactlocations/levelunknown.png"
	},
	
	-- Sword Caves
	{
		["name"] = "Wood Sword Cave",
		["code"] = "palettewoodswordcave",
		["image"] = "images/compactlocations/woodswordcave.png"
	},
	
	{
		["name"] = "White Sword Cave",
		["code"] = "palettewhiteswordcave",
		["image"] = "images/compactlocations/whiteswordcave.png"
	},

	{
		["name"] = "Magic Sword Cave",
		["code"] = "palettemagicswordcave",
		["image"] = "images/compactlocations/magicswordcave.png"
	},
	
	-- Free Stuff
	{
		["name"] = "Secret To Everybody",
		["code"] = "paletterupees",
		["image"] = "images/compactlocations/rupees.png"
	},

	{
		["name"] = "Take Any",
		["code"] = "palettetakeany",
		["image"] = "images/compactlocations/takeany.png"
	},
	
	{
		["name"] = "Letter Cave",
		["code"] = "paletteletter",
		["image"] = "images/compactlocations/lettercave.png"
	},
		
	-- Warps
	{
		["name"] = "Warp 1",
		["code"] = "palettewarp1",
		["image"] = "images/compactlocations/warpone.png"
	},
	
	{
		["name"] = "Warp 2",
		["code"] = "palettewarp2",
		["image"] = "images/compactlocations/warptwo.png"
	},
	
	{
		["name"] = "Warp 3",
		["code"] = "palettewarp3",
		["image"] = "images/compactlocations/warpthree.png"
	},
	
	{
		["name"] = "Warp 4",
		["code"] = "palettewarp4",
		["image"] = "images/compactlocations/warpfour.png"
	},
	
	-- Shops
	{
		["name"] = "Arrow Shop",
		["code"] = "paletteshoparrow",
		["image"] = "images/compactlocations/shoparrow.png"
	},
	
	{
		["name"] = "Bait Shop",
		["code"] = "paletteshopbait",
		["image"] = "images/compactlocations/shopbait.png"
	},
	
	{
		["name"] = "Blue Ring Shop",
		["code"] = "paletteshopbluering",
		["image"] = "images/compactlocations/shopbluering.png"
	},
	
	{
		["name"] = "Bomb Shop",
		["code"] = "paletteshopbomb",
		["image"] = "images/compactlocations/shopbomb.png"
	},
	
	{
		["name"] = "Candle Shop",
		["code"] = "paletteshopcandle",
		["image"] = "images/compactlocations/shopcandle.png"
	},
	
	{
		["name"] = "Hint Shop",
		["code"] = "paletteshophint",
		["image"] = "images/compactlocations/shophint.png"
	},
	
	{
		["name"] = "Key Shop",
		["code"] = "paletteshopkey",
		["image"] = "images/compactlocations/shopkey.png"
	},
	
	{
		["name"] = "Magic Shield Shop",
		["code"] = "paletteshopmagicshield",
		["image"] = "images/compactlocations/shopmagicshield.png"
	},
	
	{
		["name"] = "Medicine Shop",
		["code"] = "paletteshopmedicine",
		["image"] = "images/compactlocations/shopmedicine.png"
	},
	
	{
		["name"] = "Money Making Game",
		["code"] = "palettemmg",
		["image"] = "images/compactlocations/moneymakinggame.png"
	},
}

local BlankImage
local CurrentPalette

PaletteItem = CustomItem:extend()

-- PaletteItems are the mutually exclusive items that you use to control what you're putting on a map square
-- There are no properties used on this one since we do not want this item to change by using undo
-- In addition there is no handling for saving and loading since we want it to return to its default state if a save is loaded

function PaletteItem:init(name,code,image,index)
	self:createItem(name)
	self.code = code
	self.Active = false
	self.activeImage = image
	self.index = index
	self.disabledImage = ImageReference:FromImageReference(self.activeImage, "@disabled")
	self.ItemInstance.PotentialIcon = self.activeImage
	
	-- The 0 indexed item is our default state, and also the state our palette should start as
	if (index == 0) then
		BlankImage = self
		CurrentPalette = self
	end
	
	self:UpdateIcon()
end

function PaletteItem:UpdateIcon()
	if self.Active then
		self.ItemInstance.Icon = self.activeImage
	else
		self.ItemInstance.Icon = self.disabledImage
	end
end

function PaletteItem:canProvideCode(code)
    if code == self.code then
        return true
    else
        return false
    end
end

function PaletteItem:providesCode(code)
    if code == self.code and self.Active then
        return 1
    end
    return 0
end

function PaletteItem:onLeftClick()
	-- Technically this works fine without checking if the item isn't active, but this is cleaner
	if not self.Active then
		CurrentPalette.Active = false
		CurrentPalette:UpdateIcon()
		CurrentPalette = self
		self.Active = true
		-- Since we're not using setProperty we also can't use PropertyChanged() and so have to push the icon update here.
		self:UpdateIcon()
	end
end

function PaletteItem:onRightClick()
	-- We do need to double check that the user is right-clicking an active palette item before we set the palette to BlankImage
	if self.Active then
		CurrentPalette = BlankImage
		self.Active = false
		-- Since we're not using setProperty we also can't use PropertyChanged() and so have to push the icon update here
		self:UpdateIcon()
	end
end

-- Since we're not interested in saving palette state, we make the save function return nothing instead of the default empty table
function PaletteItem:save()
end

for i,data in pairs(PaletteData) do
	-- We convert the image paths to references here to make them easier to reference later
	data["image"] = ImageReference:FromPackRelativePath(data["image"])
	local newItem = PaletteItem(data["name"], data["code"], data["image"], i)
end

-- CompactMapItem is the squares that make up the map
-- Properties:
-- active - boolean if item is active or not
-- index - integer storing the index of the current palette, corresponding to its position in PaletteData
-- Variables:
-- self.RightClickImage - The image to be set to when the square is right-clicked
-- self.AssignedImage - The image that the square has currently been set to by left-clicking

CompactMapItem = CustomItem:extend()

-- Codes are incremented through passing the i value of a for loop
-- We don't need any other variables passed on init since the rest is fixed information
function CompactMapItem:init(iteration)
	self:createItem("Compact Map")
    self.code = "compactmap" .. iteration
	self:setProperty("active", false)
	self:setProperty("index", BlankImage.index)
    --self.RightClickImage = ImageReference:FromPackRelativePath(OnRightClickMapImage)
    self.AssignedImage = self:getPalette()
    self.ItemInstance.PotentialIcon = self.AssignedImage

    self:updateIcon()
end

function CompactMapItem:getPalette()
	return PaletteData[self:getIndex()]["image"]
end

function CompactMapItem:getActive()
	return self:getProperty("active")
end

function CompactMapItem:getIndex()
	return self:getProperty("index")
end

function CompactMapItem:updateIcon()
	if self:getActive() then
		self.ItemInstance.Icon = ImageReference:FromImageReference(self.AssignedImage, OnRightClickMapImage)
	else
		self.ItemInstance.Icon = self.AssignedImage
	end
end

function CompactMapItem:canProvideCode(code)
    if code == self.code then
        return true
    else
        return false
    end
end

function CompactMapItem:providesCode(code)
    if code == self.code and self:getActive() then
        return 1
    end
    return 0
end

function CompactMapItem:onLeftClick()
	-- If the index on our clicked square is the same as the index of our palette, set to the blank index
	if self:getIndex() == CurrentPalette.index then
		self:setProperty("index", BlankImage.index)
	-- Otherwise, set it to the index of our active palette
	else
		self:setProperty("index", CurrentPalette.index)
	end
end

-- Right clicking swaps between the item set as OnRightClickMapImage showing or not
-- Worth mentioning that right-clicking purposefully doesn't delete the underlying item
-- It's friendlier for misclicks that way
function CompactMapItem:onRightClick()
	if self:getActive() then
		self:setProperty("active", false)
	else
		self:setProperty("active", true)
	end
end

function CompactMapItem:propertyChanged(key, value)
	-- If the property changed was the index, we need to update AssignedImage used
	if key == "index" then
		self.AssignedImage = self:getPalette()
	end
    self:updateIcon()
end

-- Unlike the PaletteItems we do save the properties we've set here, we want them to show up again if someone saves/loads
function CompactMapItem:save()
    local saveData = {
		["active"] = self:getActive(),
		["index"] = self:getIndex()
	}
    return saveData
end

function CompactMapItem:load(data)
    if data ~= nil then
        self:setProperty("active", data["active"])
		self:setProperty("index", data["index"])
	end
    return true
end

for i=1,CompactMapLocationCount do
	local newItem = CompactMapItem(i)
end